function spec= calc_spec( samples, varargin )

% Set default parameter values.  These are the optimal settings according to
% the Mobisys11 results.
RATE=96000;
spec_resolution = 2^11;
band='low';
summary='p05';
TIME_STEP=0.1; % seconds per time-bin in spectrogram.
processing='post-norm-log';
window_fcn=@hamming;
samplerate=-1;
recording_len=length(samples)/RATE;

% parameters which cannot currently be changed
BINS=2^12; %the maximum spectral resolution
NFFT=(BINS-1)*2;
OVERLAP=0.0;

% parse function arguments
if( mod( length(varargin), 2 ) ~= 0 )
    error( 'An odd number of parameters was expected' );
end
for i=1:2:length(varargin)
    switch( varargin{i} )
        case 'sample_rate'
          RATE = varargin{i+1};
        case 'spec_resolution'
          spec_resolution = varargin{i+1};
        case 'band'
          band = varargin{i+1};
        case 'summary'
          summary = varargin{i+1};
        case 'time_step'
          TIME_STEP = varargin{i+1};
        case 'processing'
          processing = varargin{i+1};
        case 'downsample_rate'
          samplerate = varargin{i+1};
        case 'recording_len'
          recording_len = varargin{i+1};
        case 'window_fcn'
          window_fcn = varargin{i+1};
        otherwise
          error( [ 'Unrecognized parameter #' num2str(i+1) ] );
    end
end
if( samplerate == -1 )
    samplerate = RATE;
end

% downsample various amounts
decimation_factor = round(RATE/samplerate);
effective_rate = RATE / decimation_factor;
dec_y = decimate( samples, decimation_factor );
[tmp_spec,F]= audio_spectrogram( dec_y, effective_rate, BINS, ...
                                 TIME_STEP, OVERLAP, window_fcn );
	
% FEATURE EXTRACTION
% reduce freq resolution accordingly
clear downsampled;
new_bins = spec_resolution;
ds_factor = floor( BINS / new_bins );
if( ds_factor > 1)
    downsampled = smooth_downsample( tmp_spec, ds_factor );
    F = smooth_downsample( F, ds_factor );
else
    downsampled = tmp_spec;
end

% truncate recording
WAV_LENGTH=length(samples)/RATE;
actual_time_steps = size(downsampled,2);
downsampled = downsampled(:, 1:floor(actual_time_steps*recording_len/WAV_LENGTH));

% summarize spectrograms (matrices) in to spectra (vectors)
summary = matrix_summary( downsampled, summary );

% apply other processing
switch( processing )
  case 'log'
    processed = log(summary(2:end,:)); % first bin is zero, so can't log
  case 'diff'
    % only diff if there are more than two samples per vector
    if( size(summary,1) > 1 )
        processed = diff(summary);
    end
  case 'norm' 
    processed = normalize_spectra( summary );
  case 'norm-log' 
    processed = log( normalize_spectra( summary ) );
  case 'curve' % note that we also include norm-log here
    % constants calculated in dimension_study.m
    A = 4.11;
    mu = 1340;
    sigma = 6110;
    weights = A * exp(-((F-mu).^2) / (2*sigma^2 ));
    processed = weights .* log( normalize_spectra( summary ) );
  otherwise
    processed = summary;
end %switch


% choose the band
% NOTE: it's important that we band-pass filter AFTER normalizing, since the
% mostly-silent high frequency range is used as the reference point in
% normalization.

% These constants define the cutoff points for variound bands
clear ds_banded;
if( isstr(band) )
    switch( band )
      case 'all'
        good_freqs = find( F >= 0 );
      case 'audible'
        % discard ultrasonic frequencies
        good_freqs = find( F < 20000 );
      case 'low'
        % discard high frequencies
        good_freqs = find( F < 7000 );
      case 'verylow'
        % discard high frequencies
        good_freqs = find( F < 1000 );
      case 'veryverylow'
        % discard high frequencies
        good_freqs = find( F < 300 );
      case 'voice'
        good_freqs = find( F > 1000 & F < 7000 );
      case 'high'
        % discard low and ultrasonic frequencies
        good_freqs = find( F > 7000 & F < 20000 );
      case 'all-24'
        % discard ultrasonic frequencies
        good_freqs = find( F < 24000 );
      case 'ultrasonic'
        % discard audible frequencies
        good_freqs = find( F > 20000 );
      case '20-24_ultrasonic'
        % pick out band of interest
        good_freqs = find( F > 20000 & F < 24000 );
      case '20-48_ultrasonic'
        good_freqs = find( F > 20000 & F < 48000 );
    end    
else
    % numerically-specified band
    good_freqs = find( F >= band(1) & F < band(2) );
end
ds_banded = processed(good_freqs,:);
F = F(good_freqs);

% normalization after band-pass filtering
switch( processing )
  case 'post-norm-log' 
    spec = log( normalize_spectra( ds_banded ) );
  case 'post-curve' % note that we also include norm-log here
    % constants calculated in dimension_study.m
    A = 4.11;
    mu = 1340;
    sigma = 6110;
    weights = A * exp(-((F-mu).^2) / (2*sigma^2 ));
    spec = weights .* log( normalize_spectra( ds_banded ) );
  otherwise
    spec = ds_banded;
end